Package: bridgesampling
Type: Package
Title: Bridge Sampling for Marginal Likelihoods and Bayes Factors
Version: 1.1-2
Authors@R: c(person(given="Quentin F.", family="Gronau", role=c("aut", "cre"),
             email="Quentin.F.Gronau@gmail.com",
             comment=c(ORCID="0000-0001-5510-6943")),
             person(given="Henrik", family="Singmann", role="aut",
             comment=c(ORCID="0000-0002-4842-3657")),
             person(given="Jonathan J.", family="Forster", role="ctb"),
             person(given="Eric-Jan", family="Wagenmakers", role="ths"),
             person(family="The JASP Team", role="ctb"),
             person("Jiqiang", "Guo", role = "ctb"),
             person("Jonah", "Gabry", role = "ctb"),
             person("Ben", "Goodrich", role = c("ctb")),
             person("Kees", "Mulder", role = c("ctb")),
             person("Perry", "de Valpine", role = c("ctb"))
             )
Depends: R (>= 3.0.0)
Imports: mvtnorm, Matrix, Brobdingnag, stringr, coda, parallel, scales,
        utils, methods
Suggests: testthat, Rcpp, RcppEigen, R2jags, rjags, runjags, knitr,
        rmarkdown, R.rsp, BayesFactor, rstan, rstanarm, nimble,
        MCMCpack
Description: Provides functions for estimating marginal likelihoods, Bayes
    factors, posterior model probabilities, and normalizing constants in general,
    via different versions of bridge sampling (Meng & Wong, 1996, 
    <http://www3.stat.sinica.edu.tw/statistica/j6n4/j6n43/j6n43.htm>).
    Gronau, Singmann, & Wagenmakers (2020) <doi:10.18637/jss.v092.i10>.
License: GPL (>= 2)
LazyData: true
RoxygenNote: 7.1.1
VignetteBuilder: knitr, R.rsp
URL: https://github.com/quentingronau/bridgesampling
NeedsCompilation: no
Packaged: 2021-04-15 18:55:13 UTC; singm
Author: Quentin F. Gronau [aut, cre] (<https://orcid.org/0000-0001-5510-6943>),
  Henrik Singmann [aut] (<https://orcid.org/0000-0002-4842-3657>),
  Jonathan J. Forster [ctb],
  Eric-Jan Wagenmakers [ths],
  The JASP Team [ctb],
  Jiqiang Guo [ctb],
  Jonah Gabry [ctb],
  Ben Goodrich [ctb],
  Kees Mulder [ctb],
  Perry de Valpine [ctb]
Maintainer: Quentin F. Gronau <Quentin.F.Gronau@gmail.com>
Repository: CRAN
Date/Publication: 2021-04-16 08:50:02 UTC
Built: R 4.4.0; ; 2024-04-30 05:39:22 UTC; unix
