# Generated by roxygen2: do not edit by hand

S3method(bayes_factor,default)
S3method(bf,bridge)
S3method(bf,bridge_list)
S3method(bf,default)
S3method(bridge_sampler,MCMC_refClass)
S3method(bridge_sampler,matrix)
S3method(bridge_sampler,mcmc)
S3method(bridge_sampler,mcmc.list)
S3method(bridge_sampler,rjags)
S3method(bridge_sampler,runjags)
S3method(bridge_sampler,stanfit)
S3method(bridge_sampler,stanreg)
S3method(error_measures,bridge)
S3method(error_measures,bridge_list)
S3method(logml,bridge)
S3method(logml,bridge_list)
S3method(post_prob,bridge)
S3method(post_prob,bridge_list)
S3method(post_prob,default)
S3method(print,bf_bridge)
S3method(print,bf_bridge_list)
S3method(print,bf_default)
S3method(print,bridge)
S3method(print,bridge_list)
S3method(print,summary.bridge)
S3method(print,summary.bridge_list)
S3method(summary,bridge)
S3method(summary,bridge_list)
export(bayes_factor)
export(bf)
export(bridge_sampler)
export(error_measures)
export(logml)
export(post_prob)
import(Brobdingnag)
importFrom(Matrix,nearPD)
importFrom(coda,spectrum0.ar)
importFrom(methods,is)
importFrom(mvtnorm,dmvnorm)
importFrom(mvtnorm,rmvnorm)
importFrom(stats,cov)
importFrom(stats,dnorm)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,var)
importFrom(stringr,str_sub)
importFrom(utils,read.csv)
